program Lesson23_SphereEnviron;

// This code was created by Jeff Molofee '99
// If you've found this code useful, please let me know.
// Visit me at www.demonews.com/hosted/nehe

// Translation to Delphi/Object Pascal by Alexandre Hirzel 2003
// (based on the tranlation of lesson 18 by Brad Choate (brad@choate.net))

{ This is not the exact translation of VC++ code. As I was progressing
 through the preliminary lessons I learned by playing with and reusing my own code.
 Anyway, the procedure and function names are the same and most of the code is
 identical, in particular in the key procedures InitGL and DrawGLScene.

 The main differences are:
 1 I'm using a different way of loading texture, using a function stored in the
    unit OpenGLutils. The Glaux library is not needed, which I find more practical.
    It allows you to load bmp, jpg and tga textures.
 2 I found that spherical mapping worked well with all six objects and thus I
    kept them all (only four of them in the VC++ version, as it seems the disks
    did not work well).
 3 The keyboard checks are gathered in their own procedure "ProcessKeys"
 4 The keys work a little bit differently, allowing a better control of the
    objects positions in space. You can also control light position with the
    ASWD keys.

 A big thank to Jeff for all these wonderful tutorials. I'm very glad to be able
 to bring even a so little grain of sand to this gigantic work.

                    Alexandre Hirzel  (Alexandre.Hirzel@ie-zea.unil.ch)
}

uses
  Windows,
  Messages,
  OpenGL,
  OpenGLutils;   // Replace the Glaux library (A.H.)

{$R *.RES}

var
  h_RC	: HGLRC;	   // Permanent Rendering Context
  h_DC	: HDC; 	   // Private GDI Device Context
  keys	: array [0..255] of BOOL;		// Array Used For The Keyboard Routine
  light	: boolean;   // Lighting ON/OFF
  lp	: boolean;   // L Pressed?
  sp : bool;          // Spacebar Pressed?

  active : bool	= TRUE;	// Window Active Flag Set To TRUE By Default
  fullscreen : bool = TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default

  part1 : GLuint; 			// Start Of Disc
  part2 : GLuint; 			// End Of Disc
  p1 : GLuint = 0;			// Increase 1
  p2 : GLuint = 1;			// Increase 2

  xrot : GLfloat;			// X Rotation
  yrot : GLfloat;			// Y Rotation
  z : GLfloat = -10.0;		// Depth Into The Screen

  quadratic : GLUquadricObj;	// Storage For Our Quadratic Objects

  LightAmbient : array [0..3] of GLfloat = (0.75, 0.75, 0.75, 1.0);
  LightDiffuse : array [0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);
  LightPosition	: array [0..3] of GLfloat;

  texture : array [0..1] of GLuint;	// Storage For 2 Textures
  selectedobject : GLuint = 0;		// Which Object To Draw (NEW)

// Load Bitmaps And Convert To Textures
function LoadGLTextures:boolean;
begin
  Result:=LoadTexture('Reflect.jpg',Texture[1],GL_LINEAR,GL_LINEAR);
  Result:=LoadTexture('BG.jpg',Texture[0],GL_LINEAR,GL_LINEAR);
end;

procedure InitGL(Width: GLsizei; Height: GLsizei); // This Will Be Called Right After The GL Window Is Created
begin
  if not LoadGLTextures then exit;  // Load The Texture(s)
  glEnable(GL_TEXTURE_2D);		  // Enable Texture Mapping

  glShadeModel(GL_SMOOTH);						// Enable Smooth Shading
  glClearColor(0.0, 0.0, 0.0, 0.5);				// Black Background
  glClearDepth(1.0);							// Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);						// Enables Depth Testing
  glDepthFunc(GL_LEQUAL);						// The Type Of Depth Testing To Do
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

  glLightfv(GL_LIGHT1, GL_AMBIENT, @LightAmbient);	// Setup The Ambient Light
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @LightDiffuse);	// Setup The Diffuse Light
  glLightfv(GL_LIGHT1, GL_POSITION,@LightPosition);	// Position The Light
  glEnable(GL_LIGHT1);							// Enable Light One

  quadratic := gluNewQuadric();					// Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
  gluQuadricNormals(quadratic, GLU_SMOOTH);			// Create Smooth Normals (NEW)
  gluQuadricTexture(quadratic, True);				// Create Texture Coords (NEW)
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); // Set The Texture Generation Mode For S To Sphere Mapping (NEW)
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); // Set The Texture Generation Mode For T To Sphere Mapping (NEW)
end;

procedure ReSizeGLScene(Width: GLsizei; Height: GLsizei);
var
  fWidth, fHeight: GLfloat;

begin
  if (Height=0) then Height:=1;   // Prevent A Divide By Zero If The Window Is Too Small

  glViewport(0, 0, Width, Height);// Reset The Current Viewport And Perspective Transformation

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  fWidth := Width;
  fHeight := Height;
  gluPerspective(45.0,fWidth/fHeight,0.1,100.0);
  glMatrixMode(GL_MODELVIEW);
end;

procedure glDrawCube;
begin
  glBegin(GL_QUADS);
    // Front Face
    glNormal3f( 0.0, 0.0, 1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
    // Back Face
    glNormal3f( 0.0, 0.0,-1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
    // Top Face
    glNormal3f( 0.0, 1.0, 0.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0,  1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    // Bottom Face
    glNormal3f( 0.0,-1.0, 0.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    // Right Face
    glNormal3f( 1.0, 0.0, 0.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    // Left Face
    glNormal3f(-1.0, 0.0, 0.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
  glEnd();
end;

procedure DrawGLScene();
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT); // Clear The Screen And The Depth Buffer
  glLoadIdentity();							 // Reset The View

  glTranslatef(0.0,0.0,z);

  glEnable(GL_TEXTURE_GEN_S); 	// Enable Texture Coord Generation For S (NEW)
  glEnable(GL_TEXTURE_GEN_T); 	// Enable Texture Coord Generation For T (NEW)

  glBindTexture(GL_TEXTURE_2D, texture[1]);
  glLightfv(GL_LIGHT1, GL_POSITION,@LightPosition);	// Position The Light
  glPushMatrix();
  glRotatef(xrot, 1.0, 0.0, 0.0);
  glRotatef(yrot, 0.0, 1.0, 0.0);

  case selectedobject of
  0:    glDrawCube;
  1:    begin
          glTranslatef(0.0,0.0,-1.5);			 	// Center The Cylinder
          gluCylinder(quadratic,1.0,1.0,3.0,32,32);	// A Cylinder With A Radius Of 0.5 And A Height Of 2
        end;
  2:    gluDisk(quadratic,0.5,1.5,32,32);			// Draw A Disc (CD Shape) With An Inner Radius Of 0.5, And An Outer Radius Of 2.  Plus A Lot Of Segments ;)
  3:    gluSphere(quadratic,1.3,32,32);			// Draw A Sphere With A Radius Of 1 And 16 Longitude And 16 Latitude Segments
  4:    begin
          glTranslatef(0.0,0.0,-1.5);				// Center The Cone
          gluCylinder(quadratic,1.0,0.0,3.0,32,32);	// A Cone With A Bottom Radius Of .5 And A Height Of 2
        end;
  5:    begin
          part1 := part1 + p1;
          part2 := part2 + p2;

          if (part1>359) then					// 360 Degrees
          begin
            p1:=0;
            part1:=0;
            p2:=1;
            part2:=0;
          end;
          if (part2>359) then					// 360 Degrees
          begin
            p1:=1;
            p2:=0;
          end;
          gluPartialDisk(quadratic,0.5,1.5,32,32,part1,part2-part1);	// A Disk Like The One Before
        end;
  end;//case

  glPopMatrix();
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);

  glBindTexture(GL_TEXTURE_2D, texture[0]);	// This Will Select The BG Maps...
  glPushMatrix();
    glTranslatef(0.0, 0.0, -24.0);
    glBegin(GL_QUADS);
      glNormal3f( 0.0, 0.0, 1.0);
      glTexCoord2f(0.0, 0.0); glVertex3f(-13.3, -10.0,  10.0);
      glTexCoord2f(1.0, 0.0); glVertex3f( 13.3, -10.0,  10.0);
      glTexCoord2f(1.0, 1.0); glVertex3f( 13.3,  10.0,  10.0);
      glTexCoord2f(0.0, 1.0); glVertex3f(-13.3,  10.0,  10.0);
    glEnd();
  glPopMatrix();
end;

function WndProc(hWnd: HWND; message: UINT;
                 wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  Screen		: TRECT;
  PixelFormat	: GLuint;
const
  pfd: PIXELFORMATDESCRIPTOR = (
  nSize: sizeof( PIXELFORMATDESCRIPTOR ); // Size Of This Pixel Format Descriptor
  nVersion: 1;                // Version Number (?)
  dwFlags: PFD_DRAW_TO_WINDOW // Format Must Support Window
    or PFD_SUPPORT_OPENGL     // Format Must Support OpenGL
    or PFD_DOUBLEBUFFER;      // Must Support Double Buffering
  iPixelType: PFD_TYPE_RGBA;  // Request An RGBA Format
  cColorBits: 16;             // Select A 16Bit Color Depth
  cRedBits: 0;                // Color Bits Ignored (?)
  cRedShift: 0;
  cGreenBits: 0;
  cBlueBits: 0;
  cBlueShift: 0;
  cAlphaBits: 0;              // No Alpha Buffer
  cAlphaShift: 0;             // Shift Bit Ignored (?)
  cAccumBits: 0;              // No Accumulation Buffer
  cAccumRedBits: 0;           // Accumulation Bits Ignored (?)
  cAccumGreenBits: 0;
  cAccumBlueBits: 0;
  cAccumAlphaBits: 0;
  cDepthBits: 16;             // 16Bit Z-Buffer (Depth Buffer)
  cStencilBits: 0;            // No Stencil Buffer
  cAuxBuffers: 0;             // No Auxiliary Buffer (?)
  iLayerType: PFD_MAIN_PLANE; // Main Drawing Layer
  bReserved: 0;               // Reserved (?)
  dwLayerMask: 0;             // Layer Masks Ignored (?)
  dwVisibleMask: 0;
  dwDamageMask: 0 );
begin
  Result := 0;
  case (message) of     // Tells Windows We Want To Check The Message
  WM_CREATE:
    begin
    h_DC := GetDC(hWnd);  // Gets a Device Context For The Window
    PixelFormat := ChoosePixelFormat(h_DC, @pfd);  // Finds The Closest Match To The Pixel Format We Set Above

    if (PixelFormat=0) then
      begin
      MessageBox(0,'Cant''t Find A Suitable PixelFormat.','Error',MB_OK or MB_ICONERROR);
      PostQuitMessage(0);  // This Sends A 'Message' Telling The Program To Quit
      exit;
      end;

    if (not SetPixelFormat(h_DC,PixelFormat,@pfd)) then
      begin
      MessageBox(0,'Can''t Set The PixelFormat.','Error',MB_OK or MB_ICONERROR);
      PostQuitMessage(0);
      exit;
      end;

    h_RC := wglCreateContext(h_DC);
    if (h_RC=0) then
      begin
      MessageBox(0,'Can''t Create A GL Rendering Context.','Error',MB_OK or MB_ICONERROR);
      PostQuitMessage(0);
      exit;
      end;

    if (not wglMakeCurrent(h_DC, h_RC)) then
      begin
      MessageBox(0,'Can''t activate GLRC.','Error',MB_OK or MB_ICONERROR);
      PostQuitMessage(0);
      exit;
      end;

    GetClientRect(hWnd, Screen);
    InitGL(Screen.right, Screen.bottom);
    end;
  WM_DESTROY, WM_CLOSE:
    begin
      ChangeDisplaySettings(DEVMODE(nil^), 0);

      wglMakeCurrent(h_DC,0);
      wglDeleteContext(h_RC);
      ReleaseDC(hWnd,h_DC);

      PostQuitMessage(0);
    end;
  WM_KEYDOWN:
    begin
      keys[wParam] := TRUE;
    end;
  WM_KEYUP:
    begin
      keys[wParam] := FALSE;
    end;
  WM_SIZE:
    begin
      ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));
    end;
  else
    begin
      Result := DefWindowProc(hWnd, message, wParam, lParam);
      exit;
    end;
  end;//case
end;

procedure ProcessKeys;
begin
  if (keys[ord('L')] and not lp) then begin
    lp:=TRUE;
    light := not light;
    if (not light) then glDisable(GL_LIGHTING) else glEnable(GL_LIGHTING);
  end;
  if (not keys[ord('L')]) then lp:=FALSE;
  if (keys[VK_PRIOR]) then z := z - 0.1;
  if (keys[VK_NEXT]) then z := z + 0.1;
  if (keys[VK_UP]) then xrot:=xrot-1;
  if (keys[VK_DOWN]) then xrot:=xrot+1;
  if (keys[VK_RIGHT]) then yrot:=yrot+1;
  if (keys[VK_LEFT]) then yrot:=yrot-1;
  if (keys[Ord('A')]) then LightPosition[0]:=LightPosition[0]-1;
  if (keys[Ord('D')]) then LightPosition[0]:=LightPosition[0]+1;
  if (keys[Ord('W')]) then LightPosition[1]:=LightPosition[1]+1;
  if (keys[Ord('S')]) then LightPosition[1]:=LightPosition[1]-1;
  if (keys[32] and not(sp)) then
  begin
    sp := TRUE;
    selectedobject := selectedobject + 1;
    if (selectedobject>5) then selectedobject := 0;
  end;
  if not(keys[32]) then sp := FALSE;
end;

function WinMain(	hInstance: HINST;
			hPrevInstance: HINST;
			lpCmdLine: PChar;
			nCmdShow: integer): integer; stdcall;
var
    msg: TMsg;      // Windows Message Structure
    wc: TWndClass;  // Windows Class Structure Used To Set Up The Type Of Window
    h_Wnd: HWND;     // Storage For Window Handle
	  dmScreenSettings: DEVMODE;
begin
  ZeroMemory( @wc, sizeof( wc ) );

  wc.style		  := CS_HREDRAW or CS_VREDRAW or CS_OWNDC;
  wc.lpfnWndProc   := @WndProc;
  wc.hInstance	  := hInstance;
  wc.hCursor	  := LoadCursor(0, IDC_ARROW);
  wc.lpszClassName := 'OpenGL WinClass';

  if(RegisterClass(wc)=0) then
    begin
       MessageBox(0,'Failed To Register The Window Class.','Error',MB_OK or MB_ICONERROR);
       Result := 0;
  exit;
    end;

  h_Wnd := CreateWindow(
  'OpenGL WinClass',
  'Jeff Molofee''s GL Code Tutorial ... NeHe ''99',		// Title Appearing At The Top Of The Window

  WS_POPUP or
  WS_CLIPCHILDREN or
  WS_CLIPSIBLINGS,

  0, 0,												  // The Position Of The Window On The Screen
  640, 480, 		// The Width And Height Of The WIndow

  0,
  0,
  hInstance,
  nil);

  if (h_Wnd = 0) then
    begin
       MessageBox(0,'Window Creation Error.','Error',MB_OK or MB_ICONERROR);
       Result := 0;
       exit;
    end;

  ZeroMemory( @dmScreenSettings, sizeof( DEVMODE ) );
  dmScreenSettings.dmSize := sizeof( DEVMODE );
  dmScreenSettings.dmPelsWidth  := 640;								// Width
  dmScreenSettings.dmPelsHeight := 480;								// Height
  dmScreenSettings.dmFields     := DM_PELSWIDTH or DM_PELSHEIGHT;		// Color Depth
  ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN); // Switch To Fullscreen Mode

  ShowWindow(h_Wnd, SW_SHOW);
  UpdateWindow(h_Wnd);
  SetFocus(h_Wnd);

  while ( true ) do begin
     // Process All Messages
    while (PeekMessage(msg, 0, 0, 0, PM_NOREMOVE)) do begin
      if (GetMessage(msg, 0, 0, 0)) then begin
        TranslateMessage(msg);
        DispatchMessage(msg);
        end
      else begin
        Result := 1;
        exit;
      end;
    end;

    DrawGLScene();
    SwapBuffers(h_DC);

    if (keys[VK_ESCAPE])
    then SendMessage(h_Wnd,WM_CLOSE,0,0)
    else ProcessKeys;
  end;//while
end;

begin
  WinMain( hInstance, hPrevInst, CmdLine, CmdShow );
end.
